from .imports import *

def refreshCookie(self):
    try:
        oldSessionId = self.session.cookies["ASP.NET_SessionId"]
        #resp = self.session.get(f"https://lectio.dk/lectio/{self.skoleId}/forside.aspx", allow_redirects=False)
        headers = {
            "cookie": f"lectiogsc={self.session.cookies['lectiogsc']}; autologinkey={self.session.cookies['autologinkey']}",
        }
        resp = requests.get(f"https://www.lectio.dk/lectio/{self.skoleId}/forside.aspx", headers=headers, allow_redirects=False)
        newSessionId = resp.cookies["ASP.NET_SessionId"]
        if newSessionId != oldSessionId:
            self.session.cookies.set('ASP.NET_SessionId', newSessionId, domain='lectio.dk', path='/')

            return {"success": True}
    except Exception:
        pass

    return {"success": False}


from .imports import *

def ledigeLokaler(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/SkemaAvanceret.aspx?type=aktuelleallelokaler&nosubnav=1&prevurl=FindSkemaAdv.aspx")
    soup = BeautifulSoup(resp.text, "html.parser")

    ledigeLokalerDict = []
    for div in soup.find("div", {"id": "m_Content_LectioDetailIsland1_pa"}).find_all("div"):
        try:
            if div.get("id").startswith("printSingleControl"):
                ledigeLokalerDict.append({
                    "lokale": div.find("span").text,
                    "status": "ledigt" if "Der er ingen data..." in str(div) else "optaget"
                })
        except:
            pass

    return ledigeLokalerDict


from .imports import *
from . import _utils

colorDict = {
    "prio1.auto": "rød",
    "prio2.auto": "gul",
    "prio3.auto": "grå",
    "prio4.auto": "grøn"
}

def forside(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/forside.aspx?elevid={self.elevId}")
    soup = BeautifulSoup(resp.text, "html.parser")

    forsideDict = {
        "aktuelt": [],
        "kommunikation": {"beskeder": [], "dokumenter": []},
        "undervisning": {},
        "skema": []
    }

    for tr in soup.find("div", {"id": "s_m_Content_Content_aktueltIsland_pa"}).find_all("tr", {"class": "DashWithScroll textTop"}):
        content = tr.find("td", {"class": "infoCol"}).find("span")
        contentStr = str(content)
        for span in content.find_all("span", {"class": "bb_b"}):
            new = copy.copy(span)
            new.name = "strong"
            contentStr = contentStr.replace(str(span), str(new))

        forsideDict["aktuelt"].append({
            "punkt_farve": colorDict[tr.find("td", {"class": "iconCol"}).find("img").get("src").split("/")[-1]],
            "text": markdownify.markdownify(contentStr, heading_style="ATX").lstrip().rstrip().replace("\n\n", "\n"), #HTML til markdown det
        })


    kommunikation = soup.find("div", {"id": "s_m_Content_Content_kommIsland_pa"})
    for id, type in [["s_m_Content_Content_BeskederInfo", "beskeder"], ["s_m_Content_Content_DokumenterInfo", "dokumenter"]]:
        for tr in kommunikation.find("table", {"id": id}).find_all("tr"):
            tds = tr.find_all("td")
            try:
                forsideDict["kommunikation"][type].append({
                    "navn": tds[1].text,
                    "afsender": tds[2].find("span").get("title"),
                    "dato": tds[3].get("title"),
                    "id": re.search("id=\d+", str(tds[1].find("a").get("href"))).group().replace("id=", "")
                })
            except IndexError:
                pass

    undervisning = soup.find("div", {"id": "s_m_Content_Content_undervisningIsland_pa"})
    headings = undervisning.find_all("div", {"role": "heading"})
    tables = undervisning.find_all("table")

    for i in range(len(headings)):
        navn = headings[i].find("span", {"class": "dashboardLinkHeaderText"}).text.lower().replace(" ", "_")
        forsideDict["undervisning"][navn] = []
        for tr in tables[i].find_all("tr"):
            tds = tr.find_all("td")
            try:
                forsideDict["undervisning"][navn].append({
                    "navn": tds[1].text,
                    "dato": tds[2].get("title"),
                    "id": re.search("id=\d+", str(tds[1].find("a").get("href"))).group().replace("id=", ""),
                    "punkt_farve": colorDict[tds[0].find("img").get("src").split("/")[-1]],
                })
            except IndexError:
                pass

    for modul in soup.find("div", {"id": "s_m_Content_Content_skemaIsland_pa"}).find_all("a", {"class": "s2skemabrik"}):
        forsideDict["skema"].append(_utils.skemaBrikExtract(modul))

    return forsideDict


from .imports import *
from . import _utils

def skema(self, retry=False, uge=None, år=None, elevId=None):
    if elevId == None:
        elevId = self.elevId

    if uge == None and år == None:
        resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/SkemaNy.aspx?type=elev&elevid={elevId}")
    elif uge != None and år != None:
        uge = str(uge)
        år = str(år)
        if len(uge) == 1:
            uge = "0" + uge
        resp = self.session.get(
            f"https://www.lectio.dk/lectio/{self.skoleId}/SkemaNy.aspx?type=elev&elevid={elevId}&week={uge}{år}")
    else:
        raise Exception("Enten skal hverken uge og år være i brug ellers skal både uge og år være i brug")

    soup = BeautifulSoup(resp.text, "html.parser")

    skema = {
        "modulTider": {},
        "ugeDage": [],
        "moduler": [],
        "dagsNoter": [],
        "hold": [],
        "grupper": []
    }

    holdOgGrupper = soup.find("div", {"id": "s_m_Content_Content_holdElementLinkList"})
    for tr in holdOgGrupper.find_all("tr"):
        content = tr.find_all("li")
        if "Hold" in str(tr.find("th")):
            for hold in content:
                skema["hold"].append({"navn": hold.text, "id": hold.find("a").get("href").split("holdelementid=")[1]})
        else:
            for gruppe in content:
                skema["grupper"].append({"navn": gruppe.text, "id": gruppe.find("a").get("href").split("holdelementid=")[1]})

    for modulTid in soup.find_all("div", {"class": "s2module-info"}):
        skema["modulTider"][modulTid.prettify().split("\n")[2].lstrip()] = modulTid.prettify().split("\n")[4].lstrip()

    for dag in soup.find("tr", {"class": "s2dayHeader"}).find_all("td"):
        if dag.text != "":
            skema["ugeDage"].append(dag.text)

    i = 0
    for dagsNoter in soup.find_all("td", {"class": "s2infoHeader s2skemabrikcontainer"}):
        skema["dagsNoter"].append({
            skema["ugeDage"][i]: []
        })
        for dagsNote in dagsNoter.find_all("a"):
            skema["dagsNoter"][i][skema["ugeDage"][i]].append(dagsNote.text.lstrip())
        i += 1

    successful = False
    i = 0

    renameDictionary = {
        "Lærere": "Lærer",
        "Lokaler": "Lokale"
    }

    statusDictionary = {
        "s2brik": "normal",
        "s2cancelled": "aflyst",
        "s2changed": "ændret",

        "s2bgboxeksamen": "eksamen"
    }

    for dag in soup.find_all("div", class_="s2skemabrikcontainer"):
        if i != 0:
            dag = BeautifulSoup(str(dag), "html.parser")
            for modul in dag.find_all("a", class_="s2skemabrik"):
                modulDict = _utils.skemaBrikExtract(modul)
                skema["moduler"].append(modulDict)
        i += 1

    if len(skema) > 0:
        successful = True

    if successful:
        return skema
    elif not retry:
        self.login()
        return self.skema(retry=True)
    else:
        return False


from .imports import *

def dokumenter(self, folderid=None):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/DokumentOversigt.aspx?elevid={self.elevId}" if folderid == None else f"https://www.lectio.dk/lectio/{self.skoleId}/DokumentOversigt.aspx?elevid={self.elevId}&folderid={folderid}")
    soup = BeautifulSoup(resp.text, "html.parser")

    dokumenterDict = {"titel": soup.find("span", {"id": "s_m_Content_Content_FolderLabel"}).text, "indhold": []}

    if folderid == None:
        for element in soup.find("div", {"id": "s_m_Content_Content_FolderTreeView"}):
            if element != None:
                if str(element) != "\n":
                    itemDict = {
                        "navn": element.find("div", {"class": "TreeNode-title"}).text,
                        "type": "folder",
                        "id": element.get("lec-node-id")
                    }
                    dokumenterDict["indhold"].append(itemDict)
    else:
        _element = soup.find("div", {"lec-node-id": folderid})
        backId = _element.parent.parent.get("lec-node-id")
        dokumenterDict["indhold"].append({
            "navn": "..",
            "type": "folder",
            "id": backId if backId != None else ".."
        })

        element = _element.find("div", {"lec-role": "ltv-sublist"})
        if element != None:
            for _element in element:
                if str(_element) != "\n":
                    itemDict = {
                        "navn": _element.find("div", {"class": "TreeNode-title"}).text,
                        "type": "folder",
                        "id": _element.get("lec-node-id")
                    }
                    dokumenterDict["indhold"].append(itemDict)

        files = soup.find("div", {"id": "printfoldercontent"}).find_all("tr")
        # TODO: Tjek om mappens title er Aktiviteter eller Holdmaterialer for så er opbygningen af mappen anderledes
        if not 'class="noRecord nowrap"' in str(files):
            for tr in files[1:]:
                if dokumenterDict["titel"] == "Aktiviteter":
                    _tds = tr.find_all("td")
                    tds = [_tds[1], _tds[2], "ukendt", _tds[0]]
                    ændret_af = "ukendt"
                    id = tds[0].find("a").get('href').split("lc/")[1]
                else:
                    tds = tr.find_all("td")[:-1] if dokumenterDict["titel"] == "Nyeste dokumenter" else tr.find_all("td")[1:-1]
                    ændret_af = tds[2].find("span", {"class": "tooltip"}).get("title")
                    id = tds[0].find("a").get('href').split("documentid=")[1]

                filDict = {
                    "navn": unicodedata.normalize('NFD', tds[0].find("a").text).lstrip().rstrip(),
                    "type": "dokument",
                    "id": id,
                    "kommentar": tds[1].find("span", {"class": "tooltip"}).get("title"),
                    "ændret_af": ændret_af,
                    "dato": tds[3].text
                }

                dokumenterDict["indhold"].append(filDict)

    return dokumenterDict


from .imports import *

def generatePayload(soup, eventTarget):
    payload = {}
    for _input in soup.find("form", {"id": "aspnetForm"}).find_all("input", {"type": "hidden"}):
        id = _input.get("id")
        if id == None:
            id = _input.get("name")
        payload[id] = _input.get("value")
    payload["__EVENTTARGET"] = eventTarget

    return payload



statusDictionary = {
    "s2brik": "normal",
    "s2cancelled": "aflyst",
    "s2changed": "ændret",

    "s2bgboxeksamen": "eksamen"
}

renameDictionary = {
    "Lærere": "Lærer",
    "Lokaler": "Lokale"
}

def skemaBrikExtract(skemabrik):
    try:
        absid = re.search('absid=[0-9]+', skemabrik["href"]).group().replace("absid=", "")
    except Exception:
        absid = skemabrik.get("href")

    modulDict = {
        "navn": None,
        "tidspunkt": None,
        "hold": None,
        "lærer": None,
        "lokale": None,
        "status": "normal",
        "absid": absid,
        "andet": None
    }

    modulDetaljer = skemabrik
    statusClass = modulDetaljer.get("class")[2]
    if statusClass in statusDictionary:
        modulDict["status"] = statusDictionary[modulDetaljer.get("class")[2]]
    else:
        modulDict["status"] = modulDetaljer.get("class")[2]

    modulDetaljer = modulDetaljer["data-additionalinfo"].split("\n\n")[0].split("\n")

    for modulDetalje in modulDetaljer:
        if (value := ": ".join(modulDetalje.split(": ")[1:])) != "":
            if (navn := modulDetalje.split(": ")[0]) in renameDictionary:
                navn = renameDictionary[navn]

            modulDict[navn.lower()] = value
        else:
            try:
                int(datetime.strptime(modulDetalje.split(": ")[0].split(" til")[0],
                                      "%d/%m-%Y %H:%M").timestamp())
                modulDict["tidspunkt"] = modulDetalje
            except Exception:
                modulDict["navn"] = modulDetalje.split(": ")[0]

    try:
        modulDict["andet"] = skemabrik["data-additionalinfo"].split("\n\n")[1]
    except Exception:
        pass

    return modulDict


from .imports import *
from . import _utils

def lektier(self):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/material_lektieoversigt.aspx?elevid={self.elevId}")

    soup = BeautifulSoup(resp.text, "html.parser")

    lektier = []

    renameDictionary = {
        "Lærere": "Lærer",
        "Lokaler": "Lokale"
    }

    for tr in soup.find_all("tr"):
        modul = tr.find("a", class_="s2skemabrik")
        modulDict = _utils.skemaBrikExtract(modul)
        lektie = {
            "dato": tr.find("th").get("b"),
            "aktivitet": modulDict,
            "note": tr.find_all("td")[1].text,
            "lektier": {
                "beskrivelse": tr.find("td", {"class": "ls-homework"}).find("a").text,
                "link": tr.find("td", {"class": "ls-homework"}).find("a").get("href")
            }
        }

        lektier.append(lektie)

    return lektier


from .imports import *
from . import _utils

def beskeder(self, id=None):
    if id != None:
        if "-" in str(id):
            url = f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=&elevid={self.elevId}&selectedfolderid={id}"
        else:
            url = f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=selecthold&elevid={self.elevId}&holdid={id}"
    else:
        url = f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=liste&elevid={self.elevId}"

    resp = self.session.get(url)
    soup = BeautifulSoup(resp.text, "html.parser")

    beskederHtml = soup.find_all("tr")

    if (sideInddeling := soup.find("tr", {"class": "paging heavy"})) != None:
        payload = {}
        for _input in soup.find("form", {"id": "aspnetForm"}).find_all("input", {"type": "hidden"}):
            id = _input.get("id")
            if id == None:
                id = _input.get("name")
            payload[id] = _input.get("value")
        payload["__EVENTTARGET"] = sideInddeling.find_all("td")[-1].find("a").get("href").split("\"")[1].split("\"")[0]

        resp = self.session.post(url, data=payload)
        soup = BeautifulSoup(resp.text, "html.parser")

        beskederHtml = soup.find_all("tr")[2:-2]

    options = []
    for div in soup.find("div", {"id": "s_m_Content_Content_ListGridSelectionTree"}).find_all("div", {
        "lec-role": "treeviewnodecontainer"}):
        if "-" in div.get("lec-node-id"):
            name = div.text.rstrip()
            option = {
                "name": None,
                "id": div.get("lec-node-id"),
                "selected": False,
                "content": []
            }
            if div.find("a", {"class": "selectedFolder"}) != None:
                option["selected"] = True

            if len(name.split("\n")) == 1:
                option["name"] = name
                options.append(option)
            else:
                name = div.text.rstrip().split("\n")[0]
                option["name"] = name
                for item in div.find_all("div", {"lec-role": "treeviewnodecontainer"}):
                    option["content"].append({
                        "name": item.text.rstrip(),
                        "id": item.get("lec-node-id")
                    })

                options.append(option)

    beskedHeader = []
    for th in beskederHtml[0].find_all("th"):
        if len(header := th.text.rstrip()) != 0:
            header = header.lower().title().replace(" ", "")
            header = header[0].lower() + header[1:]
            beskedHeader.append(header)

    beskeder = []
    for beskedHtml in beskederHtml[1:]:
        besked = {}

        i = 0
        for td in beskedHtml.find_all("td"):
            if len(value := td.text.lstrip().rstrip()) != 0:
                if beskedHeader[i] == "modtagere":
                    besked[beskedHeader[i]] = td.find("span").get("title").split("\r\n")
                else:
                    try:
                        besked[beskedHeader[i]] = td.find("span").get("title")
                    except Exception:
                        besked[beskedHeader[i]] = value

                    if beskedHeader[i] == "emne":
                        print(td.find("a"))
                        besked["message_id"] = re.findall("\$_\d+", td.find("a").get("onclick"))[0].replace("$_", "")

                i += 1

        beskeder.append(besked)

    return {"besked_muligheder": options, "beskeder": beskeder}

def besked(self, message_id):
    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=showthread&elevid={self.elevId}&id={message_id}")
    soup = BeautifulSoup(resp.text, "html.parser")

    beskeder = []
    beskederHtml = soup.find("ul", {"id": "s_m_Content_Content_ThreadList"}).find_all("li")

    for besked in beskederHtml:
        bruger = besked.find("span")
        beskedDiv = None
        for div in besked.find_all("div"):
            if div.get("class") == None:
                beskedDiv = div
                break

        if beskedDiv == None:
            raise Exception("Kan ikke finde besked, rapporter venligst dette på Github")

        beskedDict = {
            "bruger": {"navn": bruger.text, "id": bruger.get("data-lectiocontextcard")},
            "titel": besked.find("h4").text,
            "dato": re.search("[0-9]+/[0-9]+-[0-9]+ [0-9]+:[0-9]+", besked.find("td").text).group(),
            "padding_left": re.search("padding-left:\d*\.?\d*", str(besked.get("style"))).group().replace("padding-left:", ""),
            "besked": markdownify.markdownify(str(beskedDiv), heading_style="ATX").lstrip().rstrip().replace("\n\n", "\n"),
            "vedhæftninger": [{"navn": vedhæft.text, "href": "https://www.lectio.dk"+vedhæft.get("href")} for vedhæft in besked.find("td").find_all("a", {"id": None})],
            "id": re.search("'\w+MESSAGE_\d+", str(besked.find("button").get("onclick"))).group().replace("'", ""),
        }
        beskeder.append(beskedDict)

    beskedDict = {
        "modtagere": soup.find("table", {"class": "ShowMessageRecipients"}).find("table", {"class": "maxWidth textTop"}).find_all("tr")[1].text.replace("Til:", "").lstrip().rstrip(),
        "beskeder": beskeder
    }

    return beskedDict

def besvarBesked(self, message_id, id, titel, content, _from):
    content = content  + "\n\n" + ["Sendt fra [url=github.com/BetterLectio/python-lectio] python-lectio [/url]", "Sendt fra [url=betterlectio.dk] Better Lectio [/url]"][_from]

    resp = self.session.get(f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=showthread&elevid={self.elevId}&id={message_id}")
    soup = BeautifulSoup(resp.text, "html.parser")

    payload = _utils.generatePayload(soup, "__Page")
    payload["__EVENTARGUMENT"] = id
    payload["__LASTFOCUS"] = ""
    payload["s$m$searchinputfield"] = ""
    payload["s$m$Content$Content$SPSearchText$tb"] = ""
    payload["LectioPostbackId"] = ""


    resp = self.session.post(f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=showthread&id={message_id}&elevid={self.elevId}", data=payload)
    soup = BeautifulSoup(resp.text, "html.parser")

    payload = _utils.generatePayload(soup, "s$m$Content$Content$CreateAnswerOKBtn")
    print(payload)
    print(urllib.parse.quote(content))
    print(urllib.parse.quote(titel))

    resp = self.session.post(f"https://www.lectio.dk/lectio/{self.skoleId}/beskeder2.aspx?type=showthread&id={message_id}&elevid={message_id}", data=f"__LASTFOCUS=&time=0&__EVENTTARGET=s%24m%24Content%24Content%24CreateAnswerOKBtn&__EVENTARGUMENT=&__SCROLLPOSITION=&__VIEWSTATEX={urllib.parse.quote(payload['__VIEWSTATEX'])}&__VIEWSTATEY_KEY=&__VIEWSTATE=&__VIEWSTATEENCRYPTED=&s%24m%24searchinputfield=&s%24m%24Content%24Content%24addRecipientToAnswerDD%24inp=&s%24m%24Content%24Content%24addRecipientToAnswerDD%24inpid=&s%24m%24Content%24Content%24Notification=NotifyBtnAuthor&s%24m%24Content%24Content%24RepliesToResponseAllowed=on&s%24m%24Content%24Content%24CreateAnswerHeading%24tb={urllib.parse.quote(titel)}&s%24m%24Content%24Content%24CreateAnswerDocChooser%24selectedDocumentId=&s%24m%24Content%24Content%24CreateAnswerContent%24TbxNAME%24tb={urllib.parse.quote(content)}&masterfootervalue=X1%21%C3%86%C3%98%C3%85&LectioPostbackId=", allow_redirects=False)

    if resp.status_code == 303:
        return {"success": True}
    else:
        raise Exception("Levering af besvarelsen var ikke succesfuld")